@@ -1,7 +1,8 @@
 /*
- * Copyright 1996-2003 by Hans Reiser, licensing governed by reiserfs/README
+ * Copyright 1996-2003 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
-  
+
 /*
  *  Reiser File System constants and structures
  */
@@ -461,7 +462,7 @@ struct reiserfs_journal_header {
 	blocksize * 8 - (block_of_super_block + 1 + 1 + 1)
 
 #define journal_default_size(block_of_super_block,blocksize) \
-	((8192 > journal_max_size (block_of_super_block,blocksize)) ? \
+	(unsigned long long)((8192 > journal_max_size (block_of_super_block,blocksize)) ? \
 		journal_max_size (block_of_super_block,blocksize) : 8192)
 
 //#define JOURNAL_DEFAULT_SIZE 8192  number of blocks in the journal
@@ -1082,9 +1083,9 @@ struct path_element  {
    in it, especially about decrement_counters_in_path(), to understand
    this structure. */
 struct path {
-  int                   path_length;                      	/* Length of the array above.   */
+  unsigned int          path_length;                      	/* Length of the array above.   */
   struct  path_element  path_elements[EXTENDED_MAX_HEIGHT];	/* Array of the path elements.  */
-  int			pos_in_item;
+  unsigned int		pos_in_item;
 };
 
 #define INITIALIZE_PATH(var) \
@@ -1548,9 +1549,9 @@ __u32 r5_hash (const char *msg, int len);
 
 
 /* node_format.c */
-int get_journal_old_start_must (reiserfs_filsys_t * fs);
-int get_journal_new_start_must (reiserfs_filsys_t * fs);
-int get_journal_start_must (reiserfs_filsys_t * fs);
+extern unsigned int get_journal_old_start_must (reiserfs_filsys_t * fs);
+extern unsigned int get_journal_new_start_must (reiserfs_filsys_t * fs);
+extern unsigned int get_journal_start_must (reiserfs_filsys_t * fs);
 /*extern hashf_t hashes [];*/
 
 
